/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2012 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#ifndef _dcrypt_h_
#define _dcrypt_h_

extern "C"
{
  /*
  ** Older compilers (GCC 4.2.1) misbehave.
  */

#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#include <gcrypt.h>
#pragma GCC diagnostic warning "-Wdeprecated-declarations"
}

#include <QMutex>
#include <QString>

class dcrypt
{
 public:
  static QStringList cipherTypes(void);
  static QStringList hashTypes(void);
  dcrypt(const QByteArray &salt,
	 const QString &cipherType,
	 const QString &hashType,
	 const QString &passphrase,
	 const int iterationCount);
  dcrypt(dcrypt *other);
  ~dcrypt();
  QByteArray decodedString(const QByteArray &byteArray, bool *ok = 0);
  QByteArray encodedString(const QByteArray &byteArray, bool *ok = 0);
  QByteArray salt(void) const;
  bool initialized(void) const;
  char *key(void) const;
  char *passphraseHash(void) const;
  int hashAlgorithm(void) const;
  size_t keyLength(void) const;
  size_t passphraseHashLength(void) const;
  unsigned long iterationCount(void) const;

 private:
  QByteArray m_salt;
  QMutex m_cipherMutex;
  bool m_initialized;
  char *m_key;
  char *m_passphraseHash;
  gcry_cipher_hd_t m_cipherHandle;
  int m_cipherAlgorithm;
  int m_hashAlgorithm;
  size_t m_keyLength;
  size_t m_passphraseHashLength;
  unsigned long m_iterationCount;
  bool openCipherHandle(void);
  bool setCipherAlgorithm(const QString &cipherType);
  bool setCipherPassphrase(const QString &passphrase);
  bool setHashAlgorithm(const QString &hashType);
  bool setInitializationVector(QByteArray &byteArray);
  void setIterationCount(const int iterationCount);
  void setSalt(const QByteArray &salt);
};

#endif
