/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011, 2012 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#ifndef _ddownloadwindow_h_
#define _ddownloadwindow_h_

#include <QMainWindow>

#include "ui_downloadWindow.h"
#include "ddownloadwindowitem.h"

class QUrl;
class QString;
class QCloseEvent;
class QProgressBar;
class QNetworkReply;

class ddownloadwindow: public QMainWindow
{
  Q_OBJECT

 public:
  ddownloadwindow(void);
  ~ddownloadwindow();
  bool isActive(void) const;
  void show(void);
  void abort(void);
  void clear(void);
  void populate(void);
  void reencode(QProgressBar *progress);
  void addUrlItem(const QUrl &url, const QString &fileName,
		  const bool isNew,
		  const QDateTime &dateTime,
		  const int choice);
  void addFileItem(const QString &srcFileName, const QString &dstFileName,
		   const bool isNew, const QDateTime &dateTime);
  void addHtmlItem(const QString &html, const QString &dstFileName);
  void closeEvent(QCloseEvent *event);
  qint64 size(void) const;

 private:
  Ui_downloadWindow ui;
  void purge(void);
  void saveState(void);
  void keyPressEvent(QKeyEvent *event);
  void createDownloadDatabase(void);

 public slots:
  void slotSetIcons(void);

 private slots:
  void slotClear(void);
  void slotClose(void);
  void slotEnterUrl(void);
  void slotClearList(void);
  void slotDownloadUrl(void);
  void slotTextChanged(const QString &text);
  void slotBitRateChanged(int state);
  void slotRecordDownload(const QString &fileName,
			  const QUrl &url,
			  const QDateTime &dateTime);
  void slotDownloadFinished(void);
  void slotCancelDownloadUrl(void);
  void slotCellDoubleClicked(int row, int col);
  void slotAuthenticationRequired(QNetworkReply *reply,
				  QAuthenticator *authenticator);
  void slotProxyAuthenticationRequired(const QNetworkProxy &proxy,
				       QAuthenticator *authenticator);

 signals:
  void saveUrl(const QUrl &url, const int choice);
  void iconsChanged(void);
};

#endif
