/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2012 Ronald Landheer-Cieslak,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#ifndef _dwebpage_h_
#define _dwebpage_h_

#include <QWebPage>

#include "dtypes.h"

class dooble;
class dexceptionswindow;

class QDateTime;

class dwebpage: public QWebPage
{
  Q_OBJECT

 public:
  dwebpage(QObject *parent = 0);
  ~dwebpage();

 private:
  QUrl m_requestedUrl;
  QNetworkAccessManager *m_networkAccessManager;
  bool javaScriptPrompt(QWebFrame *frame, const QString &msg,
			const QString &defaultValue, QString *result);
  bool javaScriptConfirm(QWebFrame *frame, const QString &msg);
  bool acceptNavigationRequest(QWebFrame *frame,
			       const QNetworkRequest &request,
			       NavigationType type);
  void downloadFavicon(const QUrl &faviconUrl, const QUrl &url);
  void javaScriptAlert(QWebFrame *frame, const QString &msg);
  void fetchFaviconPathFromFrame(QWebFrame *frame);
  dooble *findDooble(void);
  QString userAgentForUrl(const QUrl &url) const;
  QWebPage *createWindow(WebWindowType type);

 private slots:
  bool shouldInterruptJavaScript(void);
  void slotFinished(QNetworkReply *reply);
  void slotUrlChanged(const QUrl &url);
  void slotHttpToHttps(void);
  void slotFrameCreated(QWebFrame *frame);
  void slotIconDownloadFinished(void);
  void slotInitialLayoutCompleted(void);

 signals:
  void iconChanged(void);
  void loadErrorPage(const QUrl &url);
  void popupRequested(const QString &host,
		      const QUrl &url,
		      const QDateTime &dateTime);
  void exceptionRaised(dexceptionswindow *window,
		       const QUrl &url);
  void alwaysHttpsException(const QString &host,
			    const QUrl &url,
			    const QDateTime &dateTime);
};

#endif
